

// Include header files
#include "uart_sensor.h"

int8_t ret;

// Get the device pointer of the UART hardware
const struct device *uart= DEVICE_DT_GET(DT_NODELABEL(uart1));			// V - Use UART1 to read sensor =>  V - see [2]



void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data)
{	
	printk("V - Callback function was just called \n");
	switch (evt->type) {
		case UART_RX_RDY:  
			if((evt->data.rx.len) > 1){
				for (uint8_t i = 0; i < evt->data.rx.len; i++)
				{
					uart_value[i] = (evt->data.rx.buf[(evt->data.rx.offset)+i]);
					printk("V - Buffer[%d] =  %d \n", i, uart_value[i]);
				}
				memset(rx_buf, 0, sizeof(rx_buf)); // V - Erase the rx buffer for the next cycle
				comingData = 1;	
			}					
			break;
		case UART_RX_DISABLED: 
			//uart_rx_enable(dev ,rx_buf,sizeof rx_buf,RECEIVE_TIMEOUT); 
			break;
		default:
			break;
	}
}


void initialize_uart_sensor(void)
{
	printk("V - Already Initialized Asyncronous UART \n");
	
	// Verify that the UART device is ready 
	if (!device_is_ready(uart)){
		printk("UART device not ready\r\n");
		exit (1);
	}

	// Register the UART callback function
	ret = uart_callback_set(uart, uart_cb, NULL);
		if (ret) {
			printk("Fail to set callback function \r\n");
			exit (1);
		}
		
	// Start receiving by calling uart_rx_enable() and pass it the address of the receive  buffer
	ret = uart_rx_enable(uart, rx_buf,sizeof rx_buf, RECEIVE_TIMEOUT);
	if (ret) {
		printk("Fail to enable callback function \r\n");
		exit (1);
	}
	k_msleep(SLEEP_TIME_MS2);   // V - Sleep and wait for incoming Uart message (ko phải ngủ theo t của ASTAR) = 0s
}


// V- When there is coming UART data, this function returns 1 for MAIN() function reading data
uint8_t Enable_Read(void)
{
	if(comingData){
		comingData = 0;
		return (1);
	}
	else return (0);
}


uint16_t read_distance(void)
{	
	uint16_t distance = 0;
	if(((0xFF + uart_value[0] + uart_value[1]) & 0xFF) == uart_value[2])    // see [3]
	{
		distance = (uart_value[0]<<8) + uart_value[1]; // Data only contained in 2 first Byte - see [3,6]
		if(distance>280)
    	{
			return (distance);
    	}
		else{
       		printk("Below the lowest limit of sensor \n");
			return (0);
  		}
	}
}

